@@ -1,9 +1,10 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 
 from ..pool import Pool
 from .test import *
 from .model import *
+from .modelview import *
 from .mptt import *
 from .import_data import *
 from .export_data import *
@@ -53,6 +54,9 @@ def register():
         TimeDefault,
         TimeRequired,
         TimeFormat,
+        TimeDelta,
+        TimeDeltaDefault,
+        TimeDeltaRequired,
         One2One,
         One2OneTarget,
         One2OneRelation,
@@ -88,6 +92,7 @@ def register():
         Property,
         Selection,
         SelectionRequired,
+        DictSchema,
         Dict,
         DictDefault,
         DictRequired,
@@ -96,6 +101,7 @@ def register():
         BinaryRequired,
         Singleton,
         URLObject,
+        ModelStorage,
         ModelSQLRequiredField,
         ModelSQLTimestamp,
         Model4Union1,
@@ -107,6 +113,8 @@ def register():
         Model4UnionTree1,
         Model4UnionTree2,
         UnionTree,
+        ModelViewChangedValues,
+        ModelViewChangedValuesTarget,
         MPTT,
         ImportDataBoolean,
         ImportDataInteger,
@@ -150,6 +158,7 @@ def register():
         CopyMany2ManyReferenceRelation,
         Many2OneTarget,
         Many2OneDomainValidation,
+        Many2OneOrderBy,
         TestHistory,
         TestHistoryLine,
         FieldContextChild,
