@@ -1,6 +1,6 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
 # -*- coding: utf-8 -*-
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 """
 Miscelleanous tools used by tryton
 """
@@ -9,10 +9,10 @@ import sys
 import subprocess
 from threading import local
 import smtplib
-import dis
-from decimal import Decimal
 from array import array
 from itertools import islice
+import types
+
 from sql import Literal
 from sql.operators import Or
 
@@ -333,58 +333,6 @@ def reduce_ids(field, ids):
         sql.append(field.in_(discontinue_list))
     return sql
 
-_ALLOWED_CODES = set(dis.opmap[x] for x in [
-        'POP_TOP', 'ROT_TWO', 'ROT_THREE', 'ROT_FOUR', 'DUP_TOP', 'BUILD_LIST',
-        'BUILD_MAP', 'BUILD_TUPLE', 'LOAD_CONST', 'RETURN_VALUE',
-        'STORE_SUBSCR', 'UNARY_POSITIVE', 'UNARY_NEGATIVE', 'UNARY_NOT',
-        'UNARY_INVERT', 'BINARY_POWER', 'BINARY_MULTIPLY', 'BINARY_DIVIDE',
-        'BINARY_FLOOR_DIVIDE', 'BINARY_TRUE_DIVIDE', 'BINARY_MODULO',
-        'BINARY_ADD', 'BINARY_SUBTRACT', 'BINARY_LSHIFT', 'BINARY_RSHIFT',
-        'BINARY_AND', 'BINARY_XOR', 'BINARY_OR', 'STORE_MAP', 'LOAD_NAME',
-        'CALL_FUNCTION', 'COMPARE_OP', 'LOAD_ATTR', 'STORE_NAME', 'GET_ITER',
-        'FOR_ITER', 'LIST_APPEND', 'JUMP_ABSOLUTE', 'DELETE_NAME',
-        'JUMP_IF_TRUE', 'JUMP_IF_FALSE', 'JUMP_IF_FALSE_OR_POP',
-        'JUMP_IF_TRUE_OR_POP', 'POP_JUMP_IF_FALSE', 'POP_JUMP_IF_TRUE',
-        'BINARY_SUBSCR', 'JUMP_FORWARD',
-        ] if x in dis.opmap)
-
-
-@memoize(1000)
-def _compile_source(source):
-    comp = compile(source, '', 'eval')
-    codes = []
-    co_code = comp.co_code
-    i = 0
-    while i < len(co_code):
-        code = ord(co_code[i])
-        codes.append(code)
-        if code >= dis.HAVE_ARGUMENT:
-            i += 3
-        else:
-            i += 1
-    for code in codes:
-        if code not in _ALLOWED_CODES:
-            raise ValueError('opcode %s not allowed' % dis.opname[code])
-    return comp
-
-
-def safe_eval(source, data=None):
-    if '__' in source:
-        raise ValueError('Double underscores not allowed')
-
-    comp = _compile_source(source)
-    return eval(comp, {'__builtins__': {
-        'True': True,
-        'False': False,
-        'str': str,
-        'globals': locals,
-        'locals': locals,
-        'bool': bool,
-        'dict': dict,
-        'round': round,
-        'Decimal': Decimal,
-        }}, data)
-
 
 def reduce_domain(domain):
     '''
@@ -402,10 +350,10 @@ def reduce_domain(domain):
                 (isinstance(arg, list) and
                     len(arg) > 2 and
                     arg[1] in OPERATORS)):
-            #clause
+            # clause
             result.append(arg)
         elif isinstance(arg, list) and arg:
-            #sub-domain
+            # sub-domain
             sub_domain = reduce_domain(arg)
             sub_operator = sub_domain[0]
             if sub_operator == operator:
@@ -423,3 +371,10 @@ def grouped_slice(records, count=None):
         count = Transaction().cursor.IN_MAX
     for i in xrange(0, len(records), count):
         yield islice(records, i, i + count)
+
+
+def is_instance_method(cls, method):
+    for klass in cls.__mro__:
+        type_ = klass.__dict__.get(method)
+        if type_ is not None:
+            return isinstance(type_, types.FunctionType)
